/**************************************************************** 
 * Description: P3371  dijkstra 
 代码分析：
 1、结构体和变量：
Edge 结构体用于存储边的信息，包括起点、终点、权值和下一条边的索引。
head 数组存储每个点的第一条边的索引，cnt 是边的计数器。
dis 数组存储从起点到每个点的最短距离，初始值为无穷大（INF）。
vis 数组用于标记每个点是否已被访问。

2、Dijkstra 算法：
初始化所有点的最短距离为 INF，起点的距离为 0。
使用优先队列（小根堆）来不断选择当前距离最小的点进行松弛操作。
如果找到更短的路径，则更新距离并将该点加入队列。
遍历所有边，直到所有点的最短路径都确定。
3、输入输出：

使用 cin 输入点的个数、有向边的个数、起点编号和每条边的信息。
最后输出从起点到所有点的最短路径，如果无法到达则输出 2^31-1。
 * Author: Alex Li
 * Date: 2024-08-15 22:59:07
 * LastEditTime: 2024-08-15 22:59:11
****************************************************************/
#include<bits/stdc++.h>
using namespace std;

#define maxn 10005
#define maxm 500005
#define INF  1234567890

struct Edge {
    int u, v, w, next;
} e[maxm];

int head[maxn], cnt, n, m, s, vis[maxn], dis[maxn];

struct node {
    int w, now;
    // 重载运算符，把最小的元素放在堆顶（最小堆）
    inline bool operator <(const node &x) const {
        return w > x.w; // 注意这里符号要为 '>'，表示最小堆
    }
};

priority_queue<node> q; // 优先队列，使用小根堆来实现

inline void add(int u, int v, int w) {
    e[++cnt].u = u;
    e[cnt].v = v;
    e[cnt].w = w;
    e[cnt].next = head[u];
    head[u] = cnt;
    // 使用链式前向星存储图的边
}

void dijkstra() {
    for (int i = 1; i <= n; i++) {
        dis[i] = INF; // 初始化所有点的最短距离为无穷大
    }
    dis[s] = 0; // 起点的距离为0
    q.push((node){0, s});
    
    while (!q.empty()) {
        node x = q.top();
        q.pop();
        int u = x.now;
        
        if (vis[u]) continue; // 如果该点已经被访问过，则跳过
        vis[u] = 1; // 标记该点为已访问
        
        for (int i = head[u]; i; i = e[i].next) {
            int v = e[i].v;
            if (dis[v] > dis[u] + e[i].w) {
                dis[v] = dis[u] + e[i].w; // 更新最短路径
                q.push((node){dis[v], v}); // 将更新后的点加入堆中
            }
        }
    }
}

int main() {
    cin >> n >> m >> s;
    for (int i = 1, x, y, z; i <= m; i++) {
        cin >> x >> y >> z;
        add(x, y, z);
    }
    dijkstra();
    
    for (int i = 1; i <= n; i++) {
        if (dis[i] == INF) {
            cout << 2147483647 << " "; // 如果无法到达，输出2^31 - 1,也可以用INT_MAX
        } else {
            cout << dis[i] << " "; // 输出从起点到第i个点的最短路径
        }
    }
    return 0;
}
